import argparse
import logging
import warnings

import grpc
import google.protobuf.wrappers_pb2

import minknow.rpc.device_pb2
import minknow.rpc.device_pb2_grpc
import minknow.rpc.manager_pb2
import minknow.rpc.manager_pb2_grpc
import minknow.rpc.protocol_pb2
import minknow.rpc.protocol_pb2_grpc

import pyminknow.config

LOGGER = logging.getLogger(__name__)

DESCRIPTION = """
This is a client to test a Nanopore minKNOW gene sequencing device by using its gRPC interface.
"""

USAGE = "python -m pyminknow.client"


class RpcClient:
    """
    gRPC client
    """

    # TODO timeouts
    # https://groups.google.com/forum/#!topic/grpc-io/kNBQlQjxQVU

    stub_name = None
    _stub = None

    def __init__(self, channel):
        self.channel = channel

    @staticmethod
    def get_stub(service: str, channel):
        """
        Retrieve the appropriate stub (client) class for the specified service
        """

        # Map services to stubs
        stubs = dict(
            device=minknow.rpc.device_pb2_grpc.DeviceServiceStub,
            protocol=minknow.rpc.protocol_pb2_grpc.ProtocolServiceStub,
            manager=minknow.rpc.manager_pb2_grpc.ManagerServiceStub,
        )

        Stub = stubs[service]

        stub = Stub(channel=channel)

        return stub

    @property
    def stub(self):
        """Initialise stub"""

        if not self._stub:
            self._stub = self.get_stub(self.stub_name, self.channel)

        return self._stub


class ManagerClient(RpcClient):
    """
    Manager client
    """

    stub_name = 'manager'

    def describe_host(self, **kwargs) -> dict:
        request = minknow.rpc.manager_pb2.DescribeHostRequest()
        response = self.stub.describe_host(request, **kwargs)

        # return dict(
        #     product_code=response.product_code,
        #     description=response.description,
        #     serial=response.serial,
        #     network_name=response.network_name,
        # )

        return response

    def list_devices(self, **kwargs):
        warnings.warn('DEPRECATED: use `flow_cell_positions` instead', DeprecationWarning)

        request = minknow.rpc.manager_pb2.ListDevicesRequest()
        return self.stub.list_devices(request, **kwargs)

    def flow_cell_positions(self, **kwargs) -> iter:
        request = minknow.rpc.manager_pb2.FlowCellPositionsRequest()
        for response in self.stub.flow_cell_positions(request, **kwargs):
            yield from response.positions


class ProtocolClient(RpcClient):
    """
    Protocol client
    """

    stub_name = 'protocol'

    def list_protocols(self):
        request = minknow.rpc.protocol_pb2.ListProtocolsRequest()
        return self.stub.list_protocols(request)

    def start_protocol(self, identifier: str, user_info: dict = None, args: list = None) -> str:
        """
        :param identifier: Protocol ID
        :param user_info: User input describing the protocol (keys: protocol_group_id, sample_id)
        :param args: The arguments to pass to the protocol
        :returns: Run ID
        """

        user_info = user_info or dict()

        # StringValue may be null
        _user_info = minknow.rpc.protocol_pb2.ProtocolRunUserInfo(
            protocol_group_id=google.protobuf.wrappers_pb2.StringValue(value=user_info.get('protocol_group_id')),
            sample_id=google.protobuf.wrappers_pb2.StringValue(value=user_info.get('sample_id')),
        )

        request = minknow.rpc.protocol_pb2.StartProtocolRequest(
            identifier=identifier,
            user_info=_user_info,
            args=args,
        )

        response = self.stub.start_protocol(request)

        return response.run_id

    def list_protocol_runs(self) -> list:
        request = minknow.rpc.protocol_pb2.ListProtocolRunsRequest()
        response = self.stub.list_protocol_runs(request)
        return response.run_ids

    @property
    def latest_run_id(self) -> int:
        return self.list_protocol_runs()[0]

    def get_run_info_by_id(self, run_id: str = None):
        # Get latest run ID
        if run_id is None:
            run_id = self.latest_run_id

        request = minknow.rpc.protocol_pb2.GetRunInfoRequest(run_id=run_id)
        return self.stub.get_run_info(request)


class DeviceClient(RpcClient):
    """
    Device client
    """

    stub_name = 'device'

    def get_device_state(self) -> str:
        request = minknow.rpc.device_pb2.GetDeviceStateRequest()
        response = self.stub.get_device_state(request)

        # Get human-readable state
        state_name = minknow.rpc.device_pb2.GetDeviceStateResponse.DeviceState.Name(response.device_state)
        LOGGER.info("Device status: %s => '%s'", response.device_state, state_name)

        return state_name


def get_args():
    parser = argparse.ArgumentParser(usage=USAGE, description=DESCRIPTION)

    parser.add_argument('-v', '--verbose', action='store_true', help='Debug logging')
    parser.add_argument('-o', '--host', default=pyminknow.config.DEFAULT_HOST, help='Connect to this host')
    parser.add_argument('-p', '--port', type=int, default=pyminknow.config.DEFAULT_PORT, help='Connect to this port')
    parser.add_argument('-s', '--device_state', action='store_true', help='Get device state')
    parser.add_argument('-l', '--list_protocols', action='store_true', help='List available protocols')
    parser.add_argument('-d', '--list_devices', action='store_true', help='List available devices')
    parser.add_argument('-f', '--flow_cell_positions', action='store_true',
                        help='List all known positions where flow cells can be inserted')
    parser.add_argument('-i', '--start_protocol', help='Start a protocol given by this identifier')
    parser.add_argument('-g', '--protocol_group_id', help='The group which the experiment should be held in')
    parser.add_argument('-r', '--list_protocol_runs', action='store_true',
                        help='List previously started protocol run IDs, in order of starting')

    return parser, parser.parse_args()


def connect(host: str, port: int):
    """
    Connect to the server
    """

    target = '{host}:{port}'.format(host=host, port=port)

    LOGGER.info("Connecting to '%s'...", target)

    channel = grpc.insecure_channel(target=target)

    return channel


def configure_logging(verbose: bool = False):
    logging.basicConfig(level=logging.DEBUG if verbose else logging.INFO)
    logging.captureWarnings(capture=True)


def main():
    parser, args = get_args()
    configure_logging(verbose=args.verbose)

    with connect(host=args.host, port=args.port) as channel:

        manager_client = ManagerClient(channel)

        LOGGER.info(manager_client.describe_host())

        # Device
        if args.device_state:
            client = DeviceClient(channel)
            print(client.get_device_state())

        # Protocol
        elif args.list_protocols or args.start_protocol or args.list_protocol_runs:
            client = ProtocolClient(channel)

            if args.list_protocols:
                for protocol in client.list_protocols().protocols:
                    print(protocol)
            elif args.start_protocol:
                run_id = client.start_protocol(
                    identifier=args.start_protocol,
                    user_info=dict(protocol_group_id=args.protocol_group_id),
                    args=[
                        "--fast5=on",
                        "--fast5_data", "trace_table", "fastq", "raw", "zlib_compress",
                        "--base_calling=on",
                        "--fastq=on",
                        "--barcoding_kits", "EXP-NBD114", "EXP-NBD104",
                        "--experiment_time=24"
                    ],
                )
                LOGGER.info("Started run ID: %s", run_id)
                run_info = client.get_run_info_by_id(run_id)
                print(run_info)
            elif args.list_protocol_runs:
                run_ids = client.list_protocol_runs()
                print(run_ids)
            else:
                raise ValueError('Unknown command')

        # Manager
        elif args.list_devices or args.flow_cell_positions:

            if args.list_devices:
                devices = manager_client.list_devices()
                LOGGER.info("Found %s active devices", len(devices.active))
                for device in devices.active:
                    print(device)

            elif args.flow_cell_positions:
                for item in manager_client.flow_cell_positions():
                    print(item)

            else:
                raise ValueError('Unknown command')

        else:
            parser.print_help()


if __name__ == '__main__':
    main()



import logging
import uuid
import datetime
import warnings
import os.path

from collections import OrderedDict

import google.protobuf.wrappers_pb2
import google.protobuf.timestamp_pb2

import minknow.rpc.protocol_pb2
import minknow.rpc.protocol_pb2_grpc

import pyminknow.config

LOGGER = logging.getLogger(__name__)


def build_timestamp(timestamp=None) -> google.protobuf.timestamp_pb2.Timestamp:
    """Convert Python datetime to Protobuf Timestamp"""
    # https://github.com/protocolbuffers/protobuf/issues/3986
    proto_timestamp = google.protobuf.timestamp_pb2.Timestamp()
    return proto_timestamp.FromDatetime(timestamp or datetime.datetime.utcnow())


class Run:

    def __init__(self, protocol_id: str, user_info: minknow.rpc.protocol_pb2.ProtocolRunUserInfo, args: list):
        self.state = None
        self.run_id = self.make_run_id()
        self.protocol_id = protocol_id
        self.user_info = user_info
        self.args = args
        self.start_time = datetime.datetime.utcnow()
        self.end_time = None

        LOGGER.debug("Starting run ID: %s", self.run_id)
        LOGGER.debug("Protocol group ID: %s", user_info.protocol_group_id.value)
        LOGGER.debug("Sample ID: %s", user_info.sample_id.value)

        self.start()

    def serialise_data(self):
        os.makedirs(self.output_path)

        # Generate some dummy data
        path = os.path.join(self.output_path, 'my_data.txt')
        with open(path, 'w') as file:
            file.write('Hello world!\n')

    def start(self):
        self.state = minknow.rpc.protocol_pb2.ProtocolState.PROTOCOL_RUNNING

        self.serialise_data()

        self.finish()

    def finish(self):
        self.state = minknow.rpc.protocol_pb2.ProtocolState.PROTOCOL_RUNNING
        self.end_time = datetime.datetime.utcnow()

    @property
    def output_path(self):
        return os.path.join(pyminknow.config.DATA_DIR, self.run_id)

    @classmethod
    def make_run_id(cls) -> str:
        """Generate random new run identifier"""
        return uuid.uuid4().hex

    @classmethod
    def build_user_info(cls, protocol_group_id: str, sample_id: str):
        return minknow.rpc.protocol_pb2.ProtocolRunUserInfo(
            protocol_group_id=google.protobuf.wrappers_pb2.StringValue(value=protocol_group_id),
            sample_id=google.protobuf.wrappers_pb2.StringValue(value=sample_id),
        )

    @property
    def start_time(self):
        return build_timestamp(self._start_time)

    @start_time.setter
    def start_time(self, value):
        self._start_time = value

    @property
    def end_time(self):
        return build_timestamp(self._end_time)

    @end_time.setter
    def end_time(self, value):
        self._end_time = value

    @property
    def run_info(self):
        return minknow.rpc.protocol_pb2.ProtocolRunInfo(
            protocol_id=self.protocol_id,
            args=self.args,
            output_path=self.output_path,
            state=self.state,
            start_time=self.start_time,
            end_time=self.end_time,
            user_info=self.user_info,
        )


class ProtocolService(minknow.rpc.protocol_pb2_grpc.ProtocolServiceServicer):
    """
    Protocol service
    """
    add_to_server = minknow.rpc.protocol_pb2_grpc.add_ProtocolServiceServicer_to_server
    runs = OrderedDict()

    def list_protocols(self, request, context):
        if request.force_reload:
            self.clear_protocol_cache()

        protocols = self.build_protocols()

        return minknow.rpc.protocol_pb2.ListProtocolsResponse(protocols=protocols)

    def clear_protocol_cache(self):
        raise NotImplementedError

    @staticmethod
    def build_protocols() -> list:
        return [
            minknow.rpc.protocol_pb2.ProtocolInfo(
                identifier=protocol_name,
                name=protocol_name,
                tags={
                    'flow cell': minknow.rpc.protocol_pb2.ProtocolInfo.TagValue(string_value="FLO-MIN106"),
                    'kit': minknow.rpc.protocol_pb2.ProtocolInfo.TagValue(string_value="SQK-LSK109"),
                    'experiment type': minknow.rpc.protocol_pb2.ProtocolInfo.TagValue(string_value="sequencing"),
                }
            ) for protocol_name in pyminknow.config.PROTOCOLS
        ]

    @classmethod
    def _start_protocol(cls, identifier, user_info, args):
        """Emulate a real process running"""
        LOGGER.info("Starting protocol %s (Args: %s)", identifier, args)

        run = Run(protocol_id=identifier, user_info=user_info, args=args)
        cls.runs[run.run_id] = run

        return run.run_id

    def start_protocol(self, request, context):

        run_id = self._start_protocol(identifier=request.identifier, user_info=request.user_info, args=request.args)

        return minknow.rpc.protocol_pb2.StartProtocolResponse(run_id=run_id)

    @property
    def latest_run_id(self):
        """The identifier of the most recently-started run"""
        return next(reversed(self.runs.keys()))

    @property
    def run_ids(self) -> list:
        """Chronological order (by start time ascending)"""
        return list(self.runs.keys())

    def get_run_info(self, request, context):
        run_id = request.run_id

        # If no run ID is provided, information about the most recently started protocol run is provided
        if not run_id:
            run_id = self.latest_run_id

        run = self.runs[run_id]

        return run.run_info

    def set_sample_id(self, request, context):
        """
        Specify the sample identifier to set for the next protocol (deprecated)
        """

        warnings.warn('The sample_id should be set in the request when a protocol starts', DeprecationWarning)

        LOGGER.debug('Sample ID: %s', request.sample_id)

        return minknow.rpc.protocol_pb2.SetSampleIdResponse()

    def list_protocol_runs(self, request, context):
        """List previously started protocol run ids (including any current protocol), in order of starting."""
        return minknow.rpc.protocol_pb2.ListProtocolRunsResponse(run_ids=self.run_ids)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from minknow.rpc import acquisition_pb2 as minknow_dot_rpc_dot_acquisition__pb2


class AcquisitionServiceStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.start = channel.unary_unary(
        '/ont.rpc.acquisition.AcquisitionService/start',
        request_serializer=minknow_dot_rpc_dot_acquisition__pb2.StartRequest.SerializeToString,
        response_deserializer=minknow_dot_rpc_dot_acquisition__pb2.StartResponse.FromString,
        )
    self.stop = channel.unary_unary(
        '/ont.rpc.acquisition.AcquisitionService/stop',
        request_serializer=minknow_dot_rpc_dot_acquisition__pb2.StopRequest.SerializeToString,
        response_deserializer=minknow_dot_rpc_dot_acquisition__pb2.StopResponse.FromString,
        )
    self.watch_for_status_change = channel.stream_stream(
        '/ont.rpc.acquisition.AcquisitionService/watch_for_status_change',
        request_serializer=minknow_dot_rpc_dot_acquisition__pb2.WatchForStatusChangeRequest.SerializeToString,
        response_deserializer=minknow_dot_rpc_dot_acquisition__pb2.WatchForStatusChangeResponse.FromString,
        )
    self.watch_current_acquisition_run = channel.unary_stream(
        '/ont.rpc.acquisition.AcquisitionService/watch_current_acquisition_run',
        request_serializer=minknow_dot_rpc_dot_acquisition__pb2.WatchCurrentAcquisitionRunRequest.SerializeToString,
        response_deserializer=minknow_dot_rpc_dot_acquisition__pb2.AcquisitionRunInfo.FromString,
        )
    self.current_status = channel.unary_unary(
        '/ont.rpc.acquisition.AcquisitionService/current_status',
        request_serializer=minknow_dot_rpc_dot_acquisition__pb2.CurrentStatusRequest.SerializeToString,
        response_deserializer=minknow_dot_rpc_dot_acquisition__pb2.CurrentStatusResponse.FromString,
        )
    self.get_progress = channel.unary_unary(
        '/ont.rpc.acquisition.AcquisitionService/get_progress',
        request_serializer=minknow_dot_rpc_dot_acquisition__pb2.GetProgressRequest.SerializeToString,
        response_deserializer=minknow_dot_rpc_dot_acquisition__pb2.GetProgressResponse.FromString,
        )
    self.get_acquisition_info = channel.unary_unary(
        '/ont.rpc.acquisition.AcquisitionService/get_acquisition_info',
        request_serializer=minknow_dot_rpc_dot_acquisition__pb2.GetAcquisitionRunInfoRequest.SerializeToString,
        response_deserializer=minknow_dot_rpc_dot_acquisition__pb2.AcquisitionRunInfo.FromString,
        )
    self.list_acquisition_runs = channel.unary_unary(
        '/ont.rpc.acquisition.AcquisitionService/list_acquisition_runs',
        request_serializer=minknow_dot_rpc_dot_acquisition__pb2.ListAcquisitionRunsRequest.SerializeToString,
        response_deserializer=minknow_dot_rpc_dot_acquisition__pb2.ListAcquisitionRunsResponse.FromString,
        )
    self.get_current_acquisition_run = channel.unary_unary(
        '/ont.rpc.acquisition.AcquisitionService/get_current_acquisition_run',
        request_serializer=minknow_dot_rpc_dot_acquisition__pb2.GetCurrentAcquisitionRunRequest.SerializeToString,
        response_deserializer=minknow_dot_rpc_dot_acquisition__pb2.AcquisitionRunInfo.FromString,
        )
    self.set_signal_reader = channel.unary_unary(
        '/ont.rpc.acquisition.AcquisitionService/set_signal_reader',
        request_serializer=minknow_dot_rpc_dot_acquisition__pb2.SetSignalReaderRequest.SerializeToString,
        response_deserializer=minknow_dot_rpc_dot_acquisition__pb2.SetSignalReaderResponse.FromString,
        )


class AcquisitionServiceServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def start(self, request, context):
    """Starts reading data from the device

    Some setup calls will need to be made before starting data acquisition: particularly setting the analysis configuration, 
    calibration, read writer and bulk writer config and some device calls such as setting the sampling frequency

    If acqusition is already running (even in the FINISHING state), this call will fail.

    On MinIONs and GridIONs, this will enable the ASIC power supply if it is not already enabled.
    See StopRequest.keep_power_on for more details about the implications of this.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def stop(self, request, context):
    """Stops data acquisition.

    Can specify a stop mode that handles what is done with the data when data acquisition is stopped. Refer to the enum
    description for documentation on what each mode does.

    Be aware that this command will return as soon as Minknow enters the FINISHING state and not the READY state.
    So if starting a new experiment then you will have to wait for the READY state seperately
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def watch_for_status_change(self, request_iterator, context):
    """Watches for status changes within MinKNOW. Status states are defined from MinknowStatus enum.
    This is a bi-directional stream where the incoming response stream will return everytime the status has changed
    and the request stream is used to stop the watcher. Refer to http://www.grpc.io/docs/tutorials/basic/python.html
    to see how bi-directoional streaming works in grpc, but essentially when calling this function the user will have 
    to pass in a generator that will eventually yield a WatchForStatusChangeRequest(stop=True) to the cpp side.
    A wrapper class for this is provided in the Python code.

    The function will first return with the current status that MinKNOW is in. Every response thereafter will be a 
    change from one status to another.

    The ERROR_STATUS state includes errors during transition between states. If that happens, MinKNOW will 
    try to revert to the READY state. It is up to the user to determine if they wish to try to wait for MinKNOW to
    correct itself or to try some other course of action
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def watch_current_acquisition_run(self, request, context):
    """Returns current acquisition run info and streams any changes to the current acquisition

    This call can be made even if acquisition is not running. In this case, the next streamed
    response will be the start of a new acquisition and you will receive updates for that acquisition
    until it finishes.

    If an acquisition finishes this stream will still continue to run and you will be notified when a new acquisition starts.

    Note if you begin this stream before any acquisition is started in minknow the state is `ACQUISITION_COMPLETED`.

    Since 1.13
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def current_status(self, request, context):
    """Check the current status of MinKNOW.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_progress(self, request, context):
    """Information on how much data has been acquired, processed and written.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_acquisition_info(self, request, context):
    """Gets information about an acquisition run, run within this instance on MinKNOW.

    If no run ID is provided, information about the most recently started acquisition run is
    provided.

    Since 1.11
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def list_acquisition_runs(self, request, context):
    """Gets information about all previous acquisitions.

    Since 1.11
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_current_acquisition_run(self, request, context):
    """Returns the name and run id of the currently running acquisition.

    Will fail with FAILED_PRECONDITION if there is no acquisition running

    Since 1.11
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def set_signal_reader(self, request, context):
    """Specify the signal reader to use

    Since 3.6
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_AcquisitionServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'start': grpc.unary_unary_rpc_method_handler(
          servicer.start,
          request_deserializer=minknow_dot_rpc_dot_acquisition__pb2.StartRequest.FromString,
          response_serializer=minknow_dot_rpc_dot_acquisition__pb2.StartResponse.SerializeToString,
      ),
      'stop': grpc.unary_unary_rpc_method_handler(
          servicer.stop,
          request_deserializer=minknow_dot_rpc_dot_acquisition__pb2.StopRequest.FromString,
          response_serializer=minknow_dot_rpc_dot_acquisition__pb2.StopResponse.SerializeToString,
      ),
      'watch_for_status_change': grpc.stream_stream_rpc_method_handler(
          servicer.watch_for_status_change,
          request_deserializer=minknow_dot_rpc_dot_acquisition__pb2.WatchForStatusChangeRequest.FromString,
          response_serializer=minknow_dot_rpc_dot_acquisition__pb2.WatchForStatusChangeResponse.SerializeToString,
      ),
      'watch_current_acquisition_run': grpc.unary_stream_rpc_method_handler(
          servicer.watch_current_acquisition_run,
          request_deserializer=minknow_dot_rpc_dot_acquisition__pb2.WatchCurrentAcquisitionRunRequest.FromString,
          response_serializer=minknow_dot_rpc_dot_acquisition__pb2.AcquisitionRunInfo.SerializeToString,
      ),
      'current_status': grpc.unary_unary_rpc_method_handler(
          servicer.current_status,
          request_deserializer=minknow_dot_rpc_dot_acquisition__pb2.CurrentStatusRequest.FromString,
          response_serializer=minknow_dot_rpc_dot_acquisition__pb2.CurrentStatusResponse.SerializeToString,
      ),
      'get_progress': grpc.unary_unary_rpc_method_handler(
          servicer.get_progress,
          request_deserializer=minknow_dot_rpc_dot_acquisition__pb2.GetProgressRequest.FromString,
          response_serializer=minknow_dot_rpc_dot_acquisition__pb2.GetProgressResponse.SerializeToString,
      ),
      'get_acquisition_info': grpc.unary_unary_rpc_method_handler(
          servicer.get_acquisition_info,
          request_deserializer=minknow_dot_rpc_dot_acquisition__pb2.GetAcquisitionRunInfoRequest.FromString,
          response_serializer=minknow_dot_rpc_dot_acquisition__pb2.AcquisitionRunInfo.SerializeToString,
      ),
      'list_acquisition_runs': grpc.unary_unary_rpc_method_handler(
          servicer.list_acquisition_runs,
          request_deserializer=minknow_dot_rpc_dot_acquisition__pb2.ListAcquisitionRunsRequest.FromString,
          response_serializer=minknow_dot_rpc_dot_acquisition__pb2.ListAcquisitionRunsResponse.SerializeToString,
      ),
      'get_current_acquisition_run': grpc.unary_unary_rpc_method_handler(
          servicer.get_current_acquisition_run,
          request_deserializer=minknow_dot_rpc_dot_acquisition__pb2.GetCurrentAcquisitionRunRequest.FromString,
          response_serializer=minknow_dot_rpc_dot_acquisition__pb2.AcquisitionRunInfo.SerializeToString,
      ),
      'set_signal_reader': grpc.unary_unary_rpc_method_handler(
          servicer.set_signal_reader,
          request_deserializer=minknow_dot_rpc_dot_acquisition__pb2.SetSignalReaderRequest.FromString,
          response_serializer=minknow_dot_rpc_dot_acquisition__pb2.SetSignalReaderResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'ont.rpc.acquisition.AcquisitionService', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



DEFAULT_HOST = 'localhost'
DEFAULT_PORT = 9501
GRACE = 5  # seconds

PROTOCOLS = (
    'PROTOCOL_A',
    'PROTOCOL_B',
    'PROTOCOL_C',
)

DATA_DIR = '/data'

PRODUCT_CODE = 'MIN-101B-FAKE'
DESCRIPTION = 'Fake MinIT'
SERIAL = 'FAKE-0123456789'
FLOW_CELL_ID = 'FLO-MIN000'



import concurrent.futures
import logging

import grpc

import pyminknow.service.device
import pyminknow.service.manager
import pyminknow.service.protocol

LOGGER = logging.getLogger(__name__)


class Server:
    SERVICES = {
        pyminknow.service.device.DeviceService,
        pyminknow.service.protocol.ProtocolService,
        pyminknow.service.manager.ManagerService,
    }

    def __init__(self, port: int):
        self.port = port
        self.thread_pool = concurrent.futures.ThreadPoolExecutor(max_workers=10)
        self.server = grpc.server(thread_pool=self.thread_pool)
        self.server.add_insecure_port('[::]:{port}'.format(port=port))

        # Register services
        for Service in self.SERVICES:
            servicer = Service()
            servicer.add_to_server(self.server)
            LOGGER.info('Registered %s', Service.__name__)

    def start(self):
        self.server.start()
        LOGGER.info("Listening on port %s", self.port)

    def stop(self, grace: float):
        LOGGER.info('Stopping server...')
        self.server.stop(grace=grace)
        LOGGER.info("Server stopped")

    def wait(self):
        self.server.wait_for_termination()

    def serve(self, grace: float):
        """Run the server"""

        try:
            self.start()
            self.wait()
        except KeyboardInterrupt:
            self.stop(grace=grace)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from minknow.rpc import manager_pb2 as minknow_dot_rpc_dot_manager__pb2


class ManagerServiceStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.describe_host = channel.unary_unary(
        '/ont.rpc.manager.ManagerService/describe_host',
        request_serializer=minknow_dot_rpc_dot_manager__pb2.DescribeHostRequest.SerializeToString,
        response_deserializer=minknow_dot_rpc_dot_manager__pb2.DescribeHostResponse.FromString,
        )
    self.list_devices = channel.unary_unary(
        '/ont.rpc.manager.ManagerService/list_devices',
        request_serializer=minknow_dot_rpc_dot_manager__pb2.ListDevicesRequest.SerializeToString,
        response_deserializer=minknow_dot_rpc_dot_manager__pb2.ListDevicesResponse.FromString,
        )
    self.flow_cell_positions = channel.unary_stream(
        '/ont.rpc.manager.ManagerService/flow_cell_positions',
        request_serializer=minknow_dot_rpc_dot_manager__pb2.FlowCellPositionsRequest.SerializeToString,
        response_deserializer=minknow_dot_rpc_dot_manager__pb2.FlowCellPositionsResponse.FromString,
        )
    self.watch_flow_cell_positions = channel.unary_stream(
        '/ont.rpc.manager.ManagerService/watch_flow_cell_positions',
        request_serializer=minknow_dot_rpc_dot_manager__pb2.WatchFlowCellPositionsRequest.SerializeToString,
        response_deserializer=minknow_dot_rpc_dot_manager__pb2.WatchFlowCellPositionsResponse.FromString,
        )
    self.reset_position = channel.unary_unary(
        '/ont.rpc.manager.ManagerService/reset_position',
        request_serializer=minknow_dot_rpc_dot_manager__pb2.ResetPositionRequest.SerializeToString,
        response_deserializer=minknow_dot_rpc_dot_manager__pb2.ResetPositionResponse.FromString,
        )
    self.basecaller_api = channel.unary_unary(
        '/ont.rpc.manager.ManagerService/basecaller_api',
        request_serializer=minknow_dot_rpc_dot_manager__pb2.BasecallerApiRequest.SerializeToString,
        response_deserializer=minknow_dot_rpc_dot_manager__pb2.BasecallerApiResponse.FromString,
        )
    self.get_version_info = channel.unary_unary(
        '/ont.rpc.manager.ManagerService/get_version_info',
        request_serializer=minknow_dot_rpc_dot_manager__pb2.GetVersionInfoRequest.SerializeToString,
        response_deserializer=minknow_dot_rpc_dot_manager__pb2.GetVersionInfoResponse.FromString,
        )
    self.list_protocol_output_dir_files = channel.unary_stream(
        '/ont.rpc.manager.ManagerService/list_protocol_output_dir_files',
        request_serializer=minknow_dot_rpc_dot_manager__pb2.ListProtocolOutputDirFilesRequest.SerializeToString,
        response_deserializer=minknow_dot_rpc_dot_manager__pb2.ListProtocolOutputDirFilesResponse.FromString,
        )
    self.create_directory = channel.unary_unary(
        '/ont.rpc.manager.ManagerService/create_directory',
        request_serializer=minknow_dot_rpc_dot_manager__pb2.CreateDirectoryRequest.SerializeToString,
        response_deserializer=minknow_dot_rpc_dot_manager__pb2.CreateDirectoryResponse.FromString,
        )


class ManagerServiceServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def describe_host(self, request, context):
    """Get information about the machine running MinKNOW.

    Since 3.6
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def list_devices(self, request, context):
    """List all devices attached to this machine.

    Devices will be in 1 of 3 states: active, pending, inactive. Will provide port information
    for any devices that are active

    DEPRECATED: use `flow_cell_positions` (or `watch_flow_cell_positions`) instead.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def flow_cell_positions(self, request, context):
    """List all known positions where flow cells can be inserted.

    Provides a snapshot of places where users can insert flow cells. It has a streamed response
    in case there are too many positions to fit into a single response, but normally there should
    only be a single response.

    Since 3.6
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def watch_flow_cell_positions(self, request, context):
    """Watch for changes in flow cell position availability and state.

    This is like flow_cell_positions, but updates are streamed as positions come and go (eg:
    MinIONs being plugged or unplugged), or their status otherwise changes.

    Since 3.6
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def reset_position(self, request, context):
    """Reset a flow cell position.

    If the hardware is in an error state or has been removed and the software is running, it will
    stop the software (causing the position to be forgotten if the hardware was removed).

    If the hardware is not in an error state
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def basecaller_api(self, request, context):
    """Get the connection information for the basecaller API.

    This is the service that implements the minknow.rpc.basecaller interface for basecalling
    reads files.

    Since 3.5
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def get_version_info(self, request, context):
    """Current version information includes:
    - Minknow version
    - Protocols version (i.e. Bream-4 version)
    - Configuration version (i.e. Wanda version)
    - Distribution version
    - Guppy version

    See also: instance.get_version_info which provides some similar information on a per instance basis.
    Since 3.3
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def list_protocol_output_dir_files(self, request, context):
    """Returns all the files that are within the protocol output dir specified in the user_conf - or a descendant of that directory
    if `path` is supplied in the request.

    INVALID_ARGUMENT will be returned if the `path` argument in the request is not a descendant of the protocol output directory

    NOT_FOUND will be returned if the path does not exist. 'The path' being either the protocol output directory if the `path` argument is not specified
    or a valid `path` argument is specified but does not exist.

    Since 3.5
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def create_directory(self, request, context):
    """Create a directory within the protocol output directory tree.

    Errors:
    INVALID_ARGUMENT will be returned if the parent path (where the directory is requested to
    be created) is not within the protocol output directory, or if invalid characters are
    in the requested directory name.

    FAILED_PRECONDITION will be returned if the parent path is not a directory.

    NOT_FOUND will be returned if the parent path does not exist.

    ALREADY_EXISTS will be returned if the directory exists and is not a directory (if the
    directory already exists, this is treated as success).

    The possible valid parent paths can be determined using list_protocol_output_dir_files().

    Since 3.6
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_ManagerServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'describe_host': grpc.unary_unary_rpc_method_handler(
          servicer.describe_host,
          request_deserializer=minknow_dot_rpc_dot_manager__pb2.DescribeHostRequest.FromString,
          response_serializer=minknow_dot_rpc_dot_manager__pb2.DescribeHostResponse.SerializeToString,
      ),
      'list_devices': grpc.unary_unary_rpc_method_handler(
          servicer.list_devices,
          request_deserializer=minknow_dot_rpc_dot_manager__pb2.ListDevicesRequest.FromString,
          response_serializer=minknow_dot_rpc_dot_manager__pb2.ListDevicesResponse.SerializeToString,
      ),
      'flow_cell_positions': grpc.unary_stream_rpc_method_handler(
          servicer.flow_cell_positions,
          request_deserializer=minknow_dot_rpc_dot_manager__pb2.FlowCellPositionsRequest.FromString,
          response_serializer=minknow_dot_rpc_dot_manager__pb2.FlowCellPositionsResponse.SerializeToString,
      ),
      'watch_flow_cell_positions': grpc.unary_stream_rpc_method_handler(
          servicer.watch_flow_cell_positions,
          request_deserializer=minknow_dot_rpc_dot_manager__pb2.WatchFlowCellPositionsRequest.FromString,
          response_serializer=minknow_dot_rpc_dot_manager__pb2.WatchFlowCellPositionsResponse.SerializeToString,
      ),
      'reset_position': grpc.unary_unary_rpc_method_handler(
          servicer.reset_position,
          request_deserializer=minknow_dot_rpc_dot_manager__pb2.ResetPositionRequest.FromString,
          response_serializer=minknow_dot_rpc_dot_manager__pb2.ResetPositionResponse.SerializeToString,
      ),
      'basecaller_api': grpc.unary_unary_rpc_method_handler(
          servicer.basecaller_api,
          request_deserializer=minknow_dot_rpc_dot_manager__pb2.BasecallerApiRequest.FromString,
          response_serializer=minknow_dot_rpc_dot_manager__pb2.BasecallerApiResponse.SerializeToString,
      ),
      'get_version_info': grpc.unary_unary_rpc_method_handler(
          servicer.get_version_info,
          request_deserializer=minknow_dot_rpc_dot_manager__pb2.GetVersionInfoRequest.FromString,
          response_serializer=minknow_dot_rpc_dot_manager__pb2.GetVersionInfoResponse.SerializeToString,
      ),
      'list_protocol_output_dir_files': grpc.unary_stream_rpc_method_handler(
          servicer.list_protocol_output_dir_files,
          request_deserializer=minknow_dot_rpc_dot_manager__pb2.ListProtocolOutputDirFilesRequest.FromString,
          response_serializer=minknow_dot_rpc_dot_manager__pb2.ListProtocolOutputDirFilesResponse.SerializeToString,
      ),
      'create_directory': grpc.unary_unary_rpc_method_handler(
          servicer.create_directory,
          request_deserializer=minknow_dot_rpc_dot_manager__pb2.CreateDirectoryRequest.FromString,
          response_serializer=minknow_dot_rpc_dot_manager__pb2.CreateDirectoryResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'ont.rpc.manager.ManagerService', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: minknow/rpc/device.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from minknow.rpc import rpc_options_pb2 as minknow_dot_rpc_dot_rpc__options__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='minknow/rpc/device.proto',
  package='ont.rpc.device',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\x18minknow/rpc/device.proto\x12\x0eont.rpc.device\x1a\x1dminknow/rpc/rpc_options.proto\x1a\x1egoogle/protobuf/wrappers.proto\"a\n\x14\x43hannelConfiguration\x12\x0c\n\x04well\x18\x01 \x01(\r\x12\x14\n\x0ctest_current\x18\x02 \x01(\x08\x12\x14\n\x0cregeneration\x18\x03 \x01(\x08\x12\x0f\n\x07unblock\x18\x04 \x01(\x08\"\x16\n\x14GetDeviceInfoRequest\"\x8d\x04\n\x15GetDeviceInfoResponse\x12\x11\n\tdevice_id\x18\x01 \x01(\t\x12\x45\n\x0b\x64\x65vice_type\x18\x02 \x01(\x0e\x32\x30.ont.rpc.device.GetDeviceInfoResponse.DeviceType\x12\x14\n\x0cis_simulated\x18\x03 \x01(\x08\x12\x19\n\x11max_channel_count\x18\x04 \x01(\r\x12\x1d\n\x15max_wells_per_channel\x18\x05 \x01(\r\x12\x1b\n\x13\x63\x61n_set_temperature\x18\x06 \x01(\x08\x12\x14\n\x0c\x64igitisation\x18\x07 \x01(\r\x12\x18\n\x10location_defined\x18\x08 \x01(\x08\x12\x16\n\x0elocation_index\x18\t \x01(\r\x12P\n\x10\x66irmware_version\x18\n \x03(\x0b\x32\x36.ont.rpc.device.GetDeviceInfoResponse.ComponentVersion\x1a\x36\n\x10\x43omponentVersion\x12\x11\n\tcomponent\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\"[\n\nDeviceType\x12\n\n\x06MINION\x10\x00\x12\x0e\n\nPROMETHION\x10\x01\x12\x0b\n\x07GRIDION\x10\x02\x12\x13\n\x0fPROMETHION_BETA\x10\x03\x12\x0f\n\x0bMINION_MK1C\x10\x04\"\x17\n\x15GetDeviceStateRequest\"\x9c\x01\n\x16GetDeviceStateResponse\x12H\n\x0c\x64\x65vice_state\x18\x01 \x01(\x0e\x32\x32.ont.rpc.device.GetDeviceStateResponse.DeviceState\"8\n\x0b\x44\x65viceState\x12\x17\n\x13\x44\x45VICE_DISCONNECTED\x10\x00\x12\x10\n\x0c\x44\x45VICE_READY\x10\x01\"\x1a\n\x18StreamDeviceStateRequest\"\x18\n\x16GetFlowCellInfoRequest\"\xfa\x02\n\x17GetFlowCellInfoResponse\x12\x15\n\rhas_flow_cell\x18\x01 \x01(\x08\x12\x15\n\rchannel_count\x18\x02 \x01(\r\x12\x19\n\x11wells_per_channel\x18\x03 \x01(\r\x12\x14\n\x0c\x66low_cell_id\x18\x04 \x01(\t\x12\x0f\n\x07\x61sic_id\x18\x05 \x01(\r\x12\x13\n\x0b\x61sic_id_str\x18\r \x01(\t\x12\x14\n\x0cproduct_code\x18\x06 \x01(\t\x12#\n\x1buser_specified_flow_cell_id\x18\x07 \x01(\t\x12#\n\x1buser_specified_product_code\x18\x08 \x01(\t\x12\x13\n\x0bhas_adapter\x18\t \x01(\x08\x12\x12\n\nadapter_id\x18\n \x01(\t\x12\x1c\n\x12temperature_offset\x18\x0b \x01(\x02H\x00\x12\x14\n\x0c\x61sic_version\x18\x0c \x01(\tB\x1d\n\x1btemperature_offset_nullable\"\x1b\n\x19StreamFlowCellInfoRequest\"5\n!SetUserSpecifiedFlowCellIdRequest\x12\x10\n\x02id\x18\x01 \x01(\tB\x04\x88\xb5\x18\x01\"$\n\"SetUserSpecifiedFlowCellIdResponse\"8\n\"SetUserSpecifiedProductCodeRequest\x12\x12\n\x04\x63ode\x18\x01 \x01(\tB\x04\x88\xb5\x18\x01\"%\n#SetUserSpecifiedProductCodeResponse\"\x1a\n\x18GetChannelsLayoutRequest\"S\n\x19GetChannelsLayoutResponse\x12\x36\n\x0f\x63hannel_records\x18\x01 \x03(\x0b\x32\x1d.ont.rpc.device.ChannelRecord\"\xa0\x01\n\rChannelRecord\x12\n\n\x02id\x18\x01 \x01(\r\x12\x0c\n\x04name\x18\x02 \x01(\t\x12<\n\x0bmux_records\x18\x03 \x03(\x0b\x32\'.ont.rpc.device.ChannelRecord.MuxRecord\x1a\x37\n\tMuxRecord\x12\n\n\x02id\x18\x01 \x01(\r\x12\x0e\n\x06phys_x\x18\x02 \x01(\r\x12\x0e\n\x06phys_y\x18\x03 \x01(\r2\xa7\x07\n\rDeviceService\x12`\n\x0fget_device_info\x12$.ont.rpc.device.GetDeviceInfoRequest\x1a%.ont.rpc.device.GetDeviceInfoResponse\"\x00\x12\x63\n\x10get_device_state\x12%.ont.rpc.device.GetDeviceStateRequest\x1a&.ont.rpc.device.GetDeviceStateResponse\"\x00\x12k\n\x13stream_device_state\x12(.ont.rpc.device.StreamDeviceStateRequest\x1a&.ont.rpc.device.GetDeviceStateResponse\"\x00\x30\x01\x12g\n\x12get_flow_cell_info\x12&.ont.rpc.device.GetFlowCellInfoRequest\x1a\'.ont.rpc.device.GetFlowCellInfoResponse\"\x00\x12o\n\x15stream_flow_cell_info\x12).ont.rpc.device.StreamFlowCellInfoRequest\x1a\'.ont.rpc.device.GetFlowCellInfoResponse\"\x00\x30\x01\x12\x8a\x01\n\x1fset_user_specified_flow_cell_id\x12\x31.ont.rpc.device.SetUserSpecifiedFlowCellIdRequest\x1a\x32.ont.rpc.device.SetUserSpecifiedFlowCellIdResponse\"\x00\x12\x8c\x01\n\x1fset_user_specified_product_code\x12\x32.ont.rpc.device.SetUserSpecifiedProductCodeRequest\x1a\x33.ont.rpc.device.SetUserSpecifiedProductCodeResponse\"\x00\x12l\n\x13get_channels_layout\x12(.ont.rpc.device.GetChannelsLayoutRequest\x1a).ont.rpc.device.GetChannelsLayoutResponse\"\x00\x62\x06proto3'
  ,
  dependencies=[minknow_dot_rpc_dot_rpc__options__pb2.DESCRIPTOR,google_dot_protobuf_dot_wrappers__pb2.DESCRIPTOR,])



_GETDEVICEINFORESPONSE_DEVICETYPE = _descriptor.EnumDescriptor(
  name='DeviceType',
  full_name='ont.rpc.device.GetDeviceInfoResponse.DeviceType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='MINION', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROMETHION', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GRIDION', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROMETHION_BETA', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='MINION_MK1C', index=4, number=4,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=665,
  serialized_end=756,
)
_sym_db.RegisterEnumDescriptor(_GETDEVICEINFORESPONSE_DEVICETYPE)

_GETDEVICESTATERESPONSE_DEVICESTATE = _descriptor.EnumDescriptor(
  name='DeviceState',
  full_name='ont.rpc.device.GetDeviceStateResponse.DeviceState',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DEVICE_DISCONNECTED', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DEVICE_READY', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=884,
  serialized_end=940,
)
_sym_db.RegisterEnumDescriptor(_GETDEVICESTATERESPONSE_DEVICESTATE)


_CHANNELCONFIGURATION = _descriptor.Descriptor(
  name='ChannelConfiguration',
  full_name='ont.rpc.device.ChannelConfiguration',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='well', full_name='ont.rpc.device.ChannelConfiguration.well', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='test_current', full_name='ont.rpc.device.ChannelConfiguration.test_current', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='regeneration', full_name='ont.rpc.device.ChannelConfiguration.regeneration', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unblock', full_name='ont.rpc.device.ChannelConfiguration.unblock', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=107,
  serialized_end=204,
)


_GETDEVICEINFOREQUEST = _descriptor.Descriptor(
  name='GetDeviceInfoRequest',
  full_name='ont.rpc.device.GetDeviceInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=206,
  serialized_end=228,
)


_GETDEVICEINFORESPONSE_COMPONENTVERSION = _descriptor.Descriptor(
  name='ComponentVersion',
  full_name='ont.rpc.device.GetDeviceInfoResponse.ComponentVersion',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='component', full_name='ont.rpc.device.GetDeviceInfoResponse.ComponentVersion.component', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='ont.rpc.device.GetDeviceInfoResponse.ComponentVersion.version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=609,
  serialized_end=663,
)

_GETDEVICEINFORESPONSE = _descriptor.Descriptor(
  name='GetDeviceInfoResponse',
  full_name='ont.rpc.device.GetDeviceInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='device_id', full_name='ont.rpc.device.GetDeviceInfoResponse.device_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='device_type', full_name='ont.rpc.device.GetDeviceInfoResponse.device_type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='is_simulated', full_name='ont.rpc.device.GetDeviceInfoResponse.is_simulated', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_channel_count', full_name='ont.rpc.device.GetDeviceInfoResponse.max_channel_count', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='max_wells_per_channel', full_name='ont.rpc.device.GetDeviceInfoResponse.max_wells_per_channel', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='can_set_temperature', full_name='ont.rpc.device.GetDeviceInfoResponse.can_set_temperature', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='digitisation', full_name='ont.rpc.device.GetDeviceInfoResponse.digitisation', index=6,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='location_defined', full_name='ont.rpc.device.GetDeviceInfoResponse.location_defined', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='location_index', full_name='ont.rpc.device.GetDeviceInfoResponse.location_index', index=8,
      number=9, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='firmware_version', full_name='ont.rpc.device.GetDeviceInfoResponse.firmware_version', index=9,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GETDEVICEINFORESPONSE_COMPONENTVERSION, ],
  enum_types=[
    _GETDEVICEINFORESPONSE_DEVICETYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=231,
  serialized_end=756,
)


_GETDEVICESTATEREQUEST = _descriptor.Descriptor(
  name='GetDeviceStateRequest',
  full_name='ont.rpc.device.GetDeviceStateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=758,
  serialized_end=781,
)


_GETDEVICESTATERESPONSE = _descriptor.Descriptor(
  name='GetDeviceStateResponse',
  full_name='ont.rpc.device.GetDeviceStateResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='device_state', full_name='ont.rpc.device.GetDeviceStateResponse.device_state', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _GETDEVICESTATERESPONSE_DEVICESTATE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=784,
  serialized_end=940,
)


_STREAMDEVICESTATEREQUEST = _descriptor.Descriptor(
  name='StreamDeviceStateRequest',
  full_name='ont.rpc.device.StreamDeviceStateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=942,
  serialized_end=968,
)


_GETFLOWCELLINFOREQUEST = _descriptor.Descriptor(
  name='GetFlowCellInfoRequest',
  full_name='ont.rpc.device.GetFlowCellInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=970,
  serialized_end=994,
)


_GETFLOWCELLINFORESPONSE = _descriptor.Descriptor(
  name='GetFlowCellInfoResponse',
  full_name='ont.rpc.device.GetFlowCellInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='has_flow_cell', full_name='ont.rpc.device.GetFlowCellInfoResponse.has_flow_cell', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='channel_count', full_name='ont.rpc.device.GetFlowCellInfoResponse.channel_count', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='wells_per_channel', full_name='ont.rpc.device.GetFlowCellInfoResponse.wells_per_channel', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='flow_cell_id', full_name='ont.rpc.device.GetFlowCellInfoResponse.flow_cell_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='asic_id', full_name='ont.rpc.device.GetFlowCellInfoResponse.asic_id', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='asic_id_str', full_name='ont.rpc.device.GetFlowCellInfoResponse.asic_id_str', index=5,
      number=13, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='product_code', full_name='ont.rpc.device.GetFlowCellInfoResponse.product_code', index=6,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='user_specified_flow_cell_id', full_name='ont.rpc.device.GetFlowCellInfoResponse.user_specified_flow_cell_id', index=7,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='user_specified_product_code', full_name='ont.rpc.device.GetFlowCellInfoResponse.user_specified_product_code', index=8,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='has_adapter', full_name='ont.rpc.device.GetFlowCellInfoResponse.has_adapter', index=9,
      number=9, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='adapter_id', full_name='ont.rpc.device.GetFlowCellInfoResponse.adapter_id', index=10,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='temperature_offset', full_name='ont.rpc.device.GetFlowCellInfoResponse.temperature_offset', index=11,
      number=11, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='asic_version', full_name='ont.rpc.device.GetFlowCellInfoResponse.asic_version', index=12,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='temperature_offset_nullable', full_name='ont.rpc.device.GetFlowCellInfoResponse.temperature_offset_nullable',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=997,
  serialized_end=1375,
)


_STREAMFLOWCELLINFOREQUEST = _descriptor.Descriptor(
  name='StreamFlowCellInfoRequest',
  full_name='ont.rpc.device.StreamFlowCellInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1377,
  serialized_end=1404,
)


_SETUSERSPECIFIEDFLOWCELLIDREQUEST = _descriptor.Descriptor(
  name='SetUserSpecifiedFlowCellIdRequest',
  full_name='ont.rpc.device.SetUserSpecifiedFlowCellIdRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='ont.rpc.device.SetUserSpecifiedFlowCellIdRequest.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\210\265\030\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1406,
  serialized_end=1459,
)


_SETUSERSPECIFIEDFLOWCELLIDRESPONSE = _descriptor.Descriptor(
  name='SetUserSpecifiedFlowCellIdResponse',
  full_name='ont.rpc.device.SetUserSpecifiedFlowCellIdResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1461,
  serialized_end=1497,
)


_SETUSERSPECIFIEDPRODUCTCODEREQUEST = _descriptor.Descriptor(
  name='SetUserSpecifiedProductCodeRequest',
  full_name='ont.rpc.device.SetUserSpecifiedProductCodeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='ont.rpc.device.SetUserSpecifiedProductCodeRequest.code', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\210\265\030\001', file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1499,
  serialized_end=1555,
)


_SETUSERSPECIFIEDPRODUCTCODERESPONSE = _descriptor.Descriptor(
  name='SetUserSpecifiedProductCodeResponse',
  full_name='ont.rpc.device.SetUserSpecifiedProductCodeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1557,
  serialized_end=1594,
)


_GETCHANNELSLAYOUTREQUEST = _descriptor.Descriptor(
  name='GetChannelsLayoutRequest',
  full_name='ont.rpc.device.GetChannelsLayoutRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1596,
  serialized_end=1622,
)


_GETCHANNELSLAYOUTRESPONSE = _descriptor.Descriptor(
  name='GetChannelsLayoutResponse',
  full_name='ont.rpc.device.GetChannelsLayoutResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='channel_records', full_name='ont.rpc.device.GetChannelsLayoutResponse.channel_records', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1624,
  serialized_end=1707,
)


_CHANNELRECORD_MUXRECORD = _descriptor.Descriptor(
  name='MuxRecord',
  full_name='ont.rpc.device.ChannelRecord.MuxRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='ont.rpc.device.ChannelRecord.MuxRecord.id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='phys_x', full_name='ont.rpc.device.ChannelRecord.MuxRecord.phys_x', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='phys_y', full_name='ont.rpc.device.ChannelRecord.MuxRecord.phys_y', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1815,
  serialized_end=1870,
)

_CHANNELRECORD = _descriptor.Descriptor(
  name='ChannelRecord',
  full_name='ont.rpc.device.ChannelRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='ont.rpc.device.ChannelRecord.id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='ont.rpc.device.ChannelRecord.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mux_records', full_name='ont.rpc.device.ChannelRecord.mux_records', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CHANNELRECORD_MUXRECORD, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1710,
  serialized_end=1870,
)

_GETDEVICEINFORESPONSE_COMPONENTVERSION.containing_type = _GETDEVICEINFORESPONSE
_GETDEVICEINFORESPONSE.fields_by_name['device_type'].enum_type = _GETDEVICEINFORESPONSE_DEVICETYPE
_GETDEVICEINFORESPONSE.fields_by_name['firmware_version'].message_type = _GETDEVICEINFORESPONSE_COMPONENTVERSION
_GETDEVICEINFORESPONSE_DEVICETYPE.containing_type = _GETDEVICEINFORESPONSE
_GETDEVICESTATERESPONSE.fields_by_name['device_state'].enum_type = _GETDEVICESTATERESPONSE_DEVICESTATE
_GETDEVICESTATERESPONSE_DEVICESTATE.containing_type = _GETDEVICESTATERESPONSE
_GETFLOWCELLINFORESPONSE.oneofs_by_name['temperature_offset_nullable'].fields.append(
  _GETFLOWCELLINFORESPONSE.fields_by_name['temperature_offset'])
_GETFLOWCELLINFORESPONSE.fields_by_name['temperature_offset'].containing_oneof = _GETFLOWCELLINFORESPONSE.oneofs_by_name['temperature_offset_nullable']
_GETCHANNELSLAYOUTRESPONSE.fields_by_name['channel_records'].message_type = _CHANNELRECORD
_CHANNELRECORD_MUXRECORD.containing_type = _CHANNELRECORD
_CHANNELRECORD.fields_by_name['mux_records'].message_type = _CHANNELRECORD_MUXRECORD
DESCRIPTOR.message_types_by_name['ChannelConfiguration'] = _CHANNELCONFIGURATION
DESCRIPTOR.message_types_by_name['GetDeviceInfoRequest'] = _GETDEVICEINFOREQUEST
DESCRIPTOR.message_types_by_name['GetDeviceInfoResponse'] = _GETDEVICEINFORESPONSE
DESCRIPTOR.message_types_by_name['GetDeviceStateRequest'] = _GETDEVICESTATEREQUEST
DESCRIPTOR.message_types_by_name['GetDeviceStateResponse'] = _GETDEVICESTATERESPONSE
DESCRIPTOR.message_types_by_name['StreamDeviceStateRequest'] = _STREAMDEVICESTATEREQUEST
DESCRIPTOR.message_types_by_name['GetFlowCellInfoRequest'] = _GETFLOWCELLINFOREQUEST
DESCRIPTOR.message_types_by_name['GetFlowCellInfoResponse'] = _GETFLOWCELLINFORESPONSE
DESCRIPTOR.message_types_by_name['StreamFlowCellInfoRequest'] = _STREAMFLOWCELLINFOREQUEST
DESCRIPTOR.message_types_by_name['SetUserSpecifiedFlowCellIdRequest'] = _SETUSERSPECIFIEDFLOWCELLIDREQUEST
DESCRIPTOR.message_types_by_name['SetUserSpecifiedFlowCellIdResponse'] = _SETUSERSPECIFIEDFLOWCELLIDRESPONSE
DESCRIPTOR.message_types_by_name['SetUserSpecifiedProductCodeRequest'] = _SETUSERSPECIFIEDPRODUCTCODEREQUEST
DESCRIPTOR.message_types_by_name['SetUserSpecifiedProductCodeResponse'] = _SETUSERSPECIFIEDPRODUCTCODERESPONSE
DESCRIPTOR.message_types_by_name['GetChannelsLayoutRequest'] = _GETCHANNELSLAYOUTREQUEST
DESCRIPTOR.message_types_by_name['GetChannelsLayoutResponse'] = _GETCHANNELSLAYOUTRESPONSE
DESCRIPTOR.message_types_by_name['ChannelRecord'] = _CHANNELRECORD
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ChannelConfiguration = _reflection.GeneratedProtocolMessageType('ChannelConfiguration', (_message.Message,), {
  'DESCRIPTOR' : _CHANNELCONFIGURATION,
  '__module__' : 'minknow.rpc.device_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.device.ChannelConfiguration)
  })
_sym_db.RegisterMessage(ChannelConfiguration)

GetDeviceInfoRequest = _reflection.GeneratedProtocolMessageType('GetDeviceInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETDEVICEINFOREQUEST,
  '__module__' : 'minknow.rpc.device_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.device.GetDeviceInfoRequest)
  })
_sym_db.RegisterMessage(GetDeviceInfoRequest)

GetDeviceInfoResponse = _reflection.GeneratedProtocolMessageType('GetDeviceInfoResponse', (_message.Message,), {

  'ComponentVersion' : _reflection.GeneratedProtocolMessageType('ComponentVersion', (_message.Message,), {
    'DESCRIPTOR' : _GETDEVICEINFORESPONSE_COMPONENTVERSION,
    '__module__' : 'minknow.rpc.device_pb2'
    # @@protoc_insertion_point(class_scope:ont.rpc.device.GetDeviceInfoResponse.ComponentVersion)
    })
  ,
  'DESCRIPTOR' : _GETDEVICEINFORESPONSE,
  '__module__' : 'minknow.rpc.device_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.device.GetDeviceInfoResponse)
  })
_sym_db.RegisterMessage(GetDeviceInfoResponse)
_sym_db.RegisterMessage(GetDeviceInfoResponse.ComponentVersion)

GetDeviceStateRequest = _reflection.GeneratedProtocolMessageType('GetDeviceStateRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETDEVICESTATEREQUEST,
  '__module__' : 'minknow.rpc.device_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.device.GetDeviceStateRequest)
  })
_sym_db.RegisterMessage(GetDeviceStateRequest)

GetDeviceStateResponse = _reflection.GeneratedProtocolMessageType('GetDeviceStateResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETDEVICESTATERESPONSE,
  '__module__' : 'minknow.rpc.device_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.device.GetDeviceStateResponse)
  })
_sym_db.RegisterMessage(GetDeviceStateResponse)

StreamDeviceStateRequest = _reflection.GeneratedProtocolMessageType('StreamDeviceStateRequest', (_message.Message,), {
  'DESCRIPTOR' : _STREAMDEVICESTATEREQUEST,
  '__module__' : 'minknow.rpc.device_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.device.StreamDeviceStateRequest)
  })
_sym_db.RegisterMessage(StreamDeviceStateRequest)

GetFlowCellInfoRequest = _reflection.GeneratedProtocolMessageType('GetFlowCellInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETFLOWCELLINFOREQUEST,
  '__module__' : 'minknow.rpc.device_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.device.GetFlowCellInfoRequest)
  })
_sym_db.RegisterMessage(GetFlowCellInfoRequest)

GetFlowCellInfoResponse = _reflection.GeneratedProtocolMessageType('GetFlowCellInfoResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETFLOWCELLINFORESPONSE,
  '__module__' : 'minknow.rpc.device_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.device.GetFlowCellInfoResponse)
  })
_sym_db.RegisterMessage(GetFlowCellInfoResponse)

StreamFlowCellInfoRequest = _reflection.GeneratedProtocolMessageType('StreamFlowCellInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _STREAMFLOWCELLINFOREQUEST,
  '__module__' : 'minknow.rpc.device_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.device.StreamFlowCellInfoRequest)
  })
_sym_db.RegisterMessage(StreamFlowCellInfoRequest)

SetUserSpecifiedFlowCellIdRequest = _reflection.GeneratedProtocolMessageType('SetUserSpecifiedFlowCellIdRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETUSERSPECIFIEDFLOWCELLIDREQUEST,
  '__module__' : 'minknow.rpc.device_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.device.SetUserSpecifiedFlowCellIdRequest)
  })
_sym_db.RegisterMessage(SetUserSpecifiedFlowCellIdRequest)

SetUserSpecifiedFlowCellIdResponse = _reflection.GeneratedProtocolMessageType('SetUserSpecifiedFlowCellIdResponse', (_message.Message,), {
  'DESCRIPTOR' : _SETUSERSPECIFIEDFLOWCELLIDRESPONSE,
  '__module__' : 'minknow.rpc.device_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.device.SetUserSpecifiedFlowCellIdResponse)
  })
_sym_db.RegisterMessage(SetUserSpecifiedFlowCellIdResponse)

SetUserSpecifiedProductCodeRequest = _reflection.GeneratedProtocolMessageType('SetUserSpecifiedProductCodeRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETUSERSPECIFIEDPRODUCTCODEREQUEST,
  '__module__' : 'minknow.rpc.device_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.device.SetUserSpecifiedProductCodeRequest)
  })
_sym_db.RegisterMessage(SetUserSpecifiedProductCodeRequest)

SetUserSpecifiedProductCodeResponse = _reflection.GeneratedProtocolMessageType('SetUserSpecifiedProductCodeResponse', (_message.Message,), {
  'DESCRIPTOR' : _SETUSERSPECIFIEDPRODUCTCODERESPONSE,
  '__module__' : 'minknow.rpc.device_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.device.SetUserSpecifiedProductCodeResponse)
  })
_sym_db.RegisterMessage(SetUserSpecifiedProductCodeResponse)

GetChannelsLayoutRequest = _reflection.GeneratedProtocolMessageType('GetChannelsLayoutRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETCHANNELSLAYOUTREQUEST,
  '__module__' : 'minknow.rpc.device_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.device.GetChannelsLayoutRequest)
  })
_sym_db.RegisterMessage(GetChannelsLayoutRequest)

GetChannelsLayoutResponse = _reflection.GeneratedProtocolMessageType('GetChannelsLayoutResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETCHANNELSLAYOUTRESPONSE,
  '__module__' : 'minknow.rpc.device_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.device.GetChannelsLayoutResponse)
  })
_sym_db.RegisterMessage(GetChannelsLayoutResponse)

ChannelRecord = _reflection.GeneratedProtocolMessageType('ChannelRecord', (_message.Message,), {

  'MuxRecord' : _reflection.GeneratedProtocolMessageType('MuxRecord', (_message.Message,), {
    'DESCRIPTOR' : _CHANNELRECORD_MUXRECORD,
    '__module__' : 'minknow.rpc.device_pb2'
    # @@protoc_insertion_point(class_scope:ont.rpc.device.ChannelRecord.MuxRecord)
    })
  ,
  'DESCRIPTOR' : _CHANNELRECORD,
  '__module__' : 'minknow.rpc.device_pb2'
  # @@protoc_insertion_point(class_scope:ont.rpc.device.ChannelRecord)
  })
_sym_db.RegisterMessage(ChannelRecord)
_sym_db.RegisterMessage(ChannelRecord.MuxRecord)


_SETUSERSPECIFIEDFLOWCELLIDREQUEST.fields_by_name['id']._options = None
_SETUSERSPECIFIEDPRODUCTCODEREQUEST.fields_by_name['code']._options = None

_DEVICESERVICE = _descriptor.ServiceDescriptor(
  name='DeviceService',
  full_name='ont.rpc.device.DeviceService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=1873,
  serialized_end=2808,
  methods=[
  _descriptor.MethodDescriptor(
    name='get_device_info',
    full_name='ont.rpc.device.DeviceService.get_device_info',
    index=0,
    containing_service=None,
    input_type=_GETDEVICEINFOREQUEST,
    output_type=_GETDEVICEINFORESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_device_state',
    full_name='ont.rpc.device.DeviceService.get_device_state',
    index=1,
    containing_service=None,
    input_type=_GETDEVICESTATEREQUEST,
    output_type=_GETDEVICESTATERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='stream_device_state',
    full_name='ont.rpc.device.DeviceService.stream_device_state',
    index=2,
    containing_service=None,
    input_type=_STREAMDEVICESTATEREQUEST,
    output_type=_GETDEVICESTATERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_flow_cell_info',
    full_name='ont.rpc.device.DeviceService.get_flow_cell_info',
    index=3,
    containing_service=None,
    input_type=_GETFLOWCELLINFOREQUEST,
    output_type=_GETFLOWCELLINFORESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='stream_flow_cell_info',
    full_name='ont.rpc.device.DeviceService.stream_flow_cell_info',
    index=4,
    containing_service=None,
    input_type=_STREAMFLOWCELLINFOREQUEST,
    output_type=_GETFLOWCELLINFORESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='set_user_specified_flow_cell_id',
    full_name='ont.rpc.device.DeviceService.set_user_specified_flow_cell_id',
    index=5,
    containing_service=None,
    input_type=_SETUSERSPECIFIEDFLOWCELLIDREQUEST,
    output_type=_SETUSERSPECIFIEDFLOWCELLIDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='set_user_specified_product_code',
    full_name='ont.rpc.device.DeviceService.set_user_specified_product_code',
    index=6,
    containing_service=None,
    input_type=_SETUSERSPECIFIEDPRODUCTCODEREQUEST,
    output_type=_SETUSERSPECIFIEDPRODUCTCODERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='get_channels_layout',
    full_name='ont.rpc.device.DeviceService.get_channels_layout',
    index=7,
    containing_service=None,
    input_type=_GETCHANNELSLAYOUTREQUEST,
    output_type=_GETCHANNELSLAYOUTRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_DEVICESERVICE)

DESCRIPTOR.services_by_name['DeviceService'] = _DEVICESERVICE

# @@protoc_insertion_point(module_scope)



import setuptools

with open('README.md') as file:
    readme = file.read()

setuptools.setup(
    name='pyminknow',
    version='0.0.7',
    author="Joe Heffer",
    author_email="j.heffer@sheffield.ac.uk",
    description="This service mimics a Nanopore minKNOW gene sequencing device by using its gRPC interface.",
    long_description=readme,
    long_description_content_type="text/markdown",
    url="https://github.com/Joe-Heffer-Shef/pyminknow",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'grpcio~=1.27',
        'protobuf~=3.11',
        'grpcio-tools~=1.28',
    ]
)



import argparse
import logging

import pyminknow.config as config
import pyminknow.server

LOGGER = logging.getLogger(__name__)

DESCRIPTION = """
This service mimics a Nanopore minKNOW gene sequencing device by using its gRPC interface.
"""

USAGE = """
python pyminknow --help
"""


def get_args():
    """Command-line arguments"""
    parser = argparse.ArgumentParser(usage=USAGE, description=DESCRIPTION)

    parser.add_argument('-v', '--verbose', action='store_true', help='Debug logging')
    parser.add_argument('-p', '--port', type=int, default=config.DEFAULT_PORT, help='Listen on this port')
    parser.add_argument('-g', '--grace', type=int, default=config.GRACE, help='Grace period (seconds) when stopping')

    return parser.parse_args()


def configure_logging(verbose: bool = False):
    logging.basicConfig(level=logging.DEBUG if verbose else logging.INFO)
    logging.captureWarnings(capture=True)


def main():
    args = get_args()
    configure_logging(verbose=args.verbose)

    server = pyminknow.server.Server(port=args.port)
    server.serve(grace=args.grace)


if __name__ == '__main__':
    main()
